<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\AnaliseCotacoes\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class ProdutoNaoCotado extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $qr2Strategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    

    public function __construct($reportService, $entityManager, $qr2Strategy, $newCode, $environment, $connection, $dataSourceManager) {
        $this->entityManager     = $entityManager;
        $this->qr2Strategy       = $qr2Strategy;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
    }

    protected $reportMapping = array (
        
        'SUP04000' =>
            array (
                'strategy' => 'QR2',
                'parameterMapping' =>
                    array (
                        "CDFILIAL"    => "P_CDFILIAL",
                        "CDFILSOLI"   => "L_CDFILIALSOLI",
                        "CDFILSOLI1"  => "L_CDFILIALSOLI1",
                        "CDFILSOLI2"  => "L_CDFILIALSOLI2",
                        "CDFILSOLI3"  => "L_CDFILIALSOLI3",
                        "CDFILLANCA"  => "L_CDFILIALLANC",
                        "CDFILLANCA1" => "L_CDFILIALLANC1",
                        "CDFILLANCA2" => "L_CDFILIALLANC2",
                        "CDFILLANCA3" => "L_CDFILIALLANC3",
                        "CDREGIAO"    => "L_CDREGIAO",
                        "CDREGIAO1"   => "L_CDREGIAO1",
                        "CDREGIAO2"   => "L_CDREGIAO2",
                        "CDREGIAO3"   => "L_CDREGIAO3",
                        "CDOPERADOR"  => "L_CDCOMPRADOR",
                        "CDOPERADOR1" => "L_CDCOMPRADOR1",
                        "CDOPERADOR2" => "L_CDCOMPRADOR2",
                        "NRSOLICMP"   => "L_SOLICITA",
                        "NRSOLICMP1"  => "L_SOLICITA1",
                        "CDPRODINI"   => "P_CDPRODINI",
                        "CDPRODFIN"   => "P_CDPRODFIN",
                        "DTINICIAL"   => "P_DTINICIAL",
                        "DTFINAL"     => "P_DTFINAL"
                    ),
            ),
            
        'SUP04001' =>
            array (
                'strategy' => 'QR2',
                'parameterMapping' =>
                    array (
                        "CDFILIAL"    => "P_CDFILIAL",
                        "CDFILSOLI"   => "L_CDFILIALSOLI",
                        "CDFILSOLI1"  => "L_CDFILIALSOLI1",
                        "CDFILSOLI2"  => "L_CDFILIALSOLI2",
                        "CDFILSOLI3"  => "L_CDFILIALSOLI3",
                        "CDFILLANCA"  => "L_CDFILIALLANC",
                        "CDFILLANCA1" => "L_CDFILIALLANC1",
                        "CDFILLANCA2" => "L_CDFILIALLANC2",
                        "CDFILLANCA3" => "L_CDFILIALLANC3",
                        "CDREGIAO"    => "L_CDREGIAO",
                        "CDREGIAO1"   => "L_CDREGIAO1",
                        "CDREGIAO2"   => "L_CDREGIAO2",
                        "CDREGIAO3"   => "L_CDREGIAO3",
                        "CDOPERADOR"  => "L_CDCOMPRADOR",
                        "CDOPERADOR1" => "L_CDCOMPRADOR1",
                        "CDOPERADOR2" => "L_CDCOMPRADOR2",
                        "NRSOLICMP"   => "L_SOLICITA",
                        "NRSOLICMP1"  => "L_SOLICITA1",
                        "CDPRODINI"   => "P_CDPRODINI",
                        "CDPRODFIN"   => "P_CDPRODFIN",
                        "CDPRODINI1"  => "P_CDPRODINIC",
                        "CDPRODFIN1"  => "P_CDPRODFINA",
                        "DTINICIAL"   => "P_DTINICIAL",
                        "DTFINAL"     => "P_DTFINAL"
                    ),
            ),
            
            'SUP04011' =>
            array (
                'strategy' => 'QR2',
                'parameterMapping' =>
                    array (
                        "CDFILIAL"    => "P_CDFILIAL",
                        "CDFILSOLI"   => "P_CDFILIALSOLI",
                        "CDREGIAO"    => "P_CDREGIAO",
                        "CDOPERADOR"  => "P_CDCOMPRADOR",
                        "CDPRODINI"   => "P_CDPRODINI",
                        "CDPRODFIN"   => "P_CDPRODFIN",
                        "DTINICIAL"   => "P_DTINICIAL",
                        "DTFINAL"     => "P_DTFINAL",
                        "DTINICOT"    => "P_DTINICOT",
                        "DTFINCOT"    => "P_DTFINCOT",
                        "CDFILLANCA"  => "L_CDFILIALLANC",
                        "CDFILLANCA1" => "L_CDFILIALLANC1",
                        "CDFILLANCA2" => "L_CDFILIALLANC2",
                        "CDFILLANCA3" => "L_CDFILIALLANC3",
                        "NRSOLICMP"   => "L_SOLICITA",
                        "NRSOLICMP1"  => "L_SOLICITA1"
                      
                        
                    ),
            ),
    );
    
    public function insertLista($nrlista) {
    	$params = array(
    		"NRLISTA" => $nrlista,
    		"CDLISTA" => $this->environment->getCdOperador()
    	);
    	
    	$sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
   
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        
        $NRLISTA = $this->newCode->getCodeByTableName('NRLISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        
        $row = $request->getRow();
       
        $this->insertLista($NRLISTA);
        $row['__report_name'] = $row['REPORT'];
        $row['CDFILIAL'] = $this->getCdFilial();
        
        $row['CDFILSOLI1'] = $row['CDFILSOLI'];
        $row['CDFILSOLI2'] = $row['CDFILSOLI'];
        $row['CDFILSOLI3'] = $row['CDFILSOLI'];
        $row['CDFILLANCA1'] = $row['CDFILLANCA'];
        $row['CDFILLANCA2'] = $row['CDFILLANCA'];
        $row['CDFILLANCA3'] = $row['CDFILLANCA'];
        $row['CDREGIAO1'] = $row['CDREGIAO'];
        $row['CDREGIAO2'] = $row['CDREGIAO'];
        $row['CDREGIAO3'] = $row['CDREGIAO'];
        $row['CDOPERADOR1'] = $row['CDOPERADOR'];
        $row['CDOPERADOR2'] = $row['CDOPERADOR'];
        $row['NRSOLICMP1'] = $row['NRSOLICMP'];
        $row['CDPRODINI1'] = $row['CDPRODINI'];
        $row['CDPRODFIN1'] = $row['CDPRODFIN'];
        $row['DTINICIAL'] = $row['DT_ENTREGA']['start'];
        $row['DTFINAL'] = $row['DT_ENTREGA']['end'];
        $row['DTINICOT'] = $row['DT_VALIDADE']['start'];
        $row['DTFINCOT'] = $row['DT_VALIDADE']['end'];
       
        
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
    }
     public function verificaIdsisdos(DTO\Request $request, DTO\Response $response) {
        
         $params = array(
    		"NRORG" => $this->environment->getNrorg()
    	);
    	
    	$sql = "SELECT IDSISDOS FROM PARAMGERAL WHERE NRORG = :NRORG";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$resp = $query->fetchAll();

    $response->addDataSet(new DataSet("resp", $resp));
      
    }
    public function getCdFilial() {
        $query = "SELECT DISTINCT
                    NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)  AS CDFILIAL
                  FROM
                    FILIAL FIL,
                    FILIAL
                  WHERE
                    (FILIAL.CDFILIAL = :P_CDFILIAL) AND
                    NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FIL.CDFILIAL";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "P_CDFILIAL" => $this->environment->getCdFilial()
		));
        $cdfilial = $stmt->fetchAll()[0]["CDFILIAL"];
        return $cdfilial;
    }
}