<?php
namespace Teknisa\SUP\Modules\Solicitacao\Modules\Relatorios\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Sup97000VerificacaoCadastrosProcessosComprasSuprimentosService 
{
    protected $connection;
    
    public function __construct($entityManager, $environment)
    {
        $this->connection  = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    public function regiaoFilial ($row)//função serve para processo de cálculo e lançamento de compras
    {   
        $params = array(
    	    "P_CDFILIAL" => $row['CDFILIAL']
    	);
    	$sql = "SELECT 
    	            CDREGIAO 
    	        FROM 
    	            FILIAL 
    	        WHERE 
    	            CDFILIAL = :P_CDFILIAL";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
     public function verificaProdutoAtivo ($row) 
     {   
        $params = array (
    	    "P_CDPRODUTO" => $row['CDPRODUTO']
    	);
    	$sql = "SELECT 
    	            CDPRODUTO, 
    	            CDARVPROD, 
    	            NMPRODUTO, 
    	            SGUNIDADE, 
    	            IDPRODATIVPR, 
    	            VRFATOCONV, 
    	            CDPRODINTE 
    	        FROM 
    	            PRODUTO P 
    	        WHERE 
    	            (P.CDPRODUTO = :P_CDPRODUTO)";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
// Processo Cálculo da Necessidade de Compras 
    
    //abre tela região se retornar vazio
    public function FilialSemRegiao ($row)//função serve para processo de cálculo e lançamento de compras
    {   
        $params = array(
    	    "P_CDFILIAL" => $row['CDFILIAL']
    	);
    	$sql = "SELECT 
    	            FILIAL.CDFILIAL 
    	        FROM 
    	            FILIAL 
    	        WHERE 
    	            (FILIAL.CDFILIAL = :P_CDFILIAL) AND 
    	            (FILIAL.CDREGIAO IS NOT NULL) 
    	        ORDER BY 
    	            FILIAL.CDFILIAL";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    
    //abre tela filial se retornar vazio
    public function FilialSemContratoReincidido($row)//função serve para processo de cálculo e lançamento de compras
    { 
        $params = array(
    	    "P_CDFILIAL" => $row['CDFILIAL']
    	);
    	$sql = "SELECT 
    	            FILIAL.CDFILIAL 
    	        FROM 
    	            FILIAL 
    	        WHERE 
    	            (FILIAL.CDFILIAL = :P_CDFILIAL) AND 
    	            (FILIAL.IDCNTRRESC <> 'S') 
    	        ORDER BY 
    	            FILIAL.CDFILIAL";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    //abre tela produtos por unidade se retornar vazio
    public function ProdutoPorFilial($row)//função serve para processo de cálculo e lançamento de compras
    {   
        $params = array(
    	    "P_CDFILIAL"  => $row['CDFILIAL'],
    	    "P_CDPRODUTO" => $row['CDPRODUTO']
    	);
    	$sql = "SELECT 
    	            P.CDFILIAL, 
    	            P.CDPRODUTO, 
    	            PR.NMPRODUTO 
    	        FROM 
    	            PRODFILI P, 
    	            PRODUTO PR 
    	        WHERE 
    	            (P.CDFILIAL      = :P_CDFILIAL)  AND 
    	            (P.CDPRODUTO     = PR.CDPRODUTO) AND 
    	            (PR.IDPRODATIVPR = 'S')          AND 
    	            (PR.CDPRODUTO    = :P_CDPRODUTO) AND 
    	            (P.IDPRODATIV    = 'A') 
    	        ORDER BY 
    	            PR.CDARVPROD";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    //abre tela Dados de Produtos na Filial se retornar vazio
    public function DadosProdutoFilial($row) 
    {
        $params = array(
    	    "P_CDFILIAL"  => $row['CDFILIAL'],
    	    "P_CDPRODUTO" => $row['CDARVPROD']
    	);
    	$sql = "SELECT 
    	            PERFILPROD.CDPERIENTR, 
    	            PERFILPROD.CDPRODUTO 
    	        FROM 
    	            PERFILPROD, 
    	            PERIENTR 
    	        WHERE 
    	            PERFILPROD.CDFILIAL   = :P_CDFILIAL AND 
    	            PERFILPROD.CDPERIENTR = PERIENTR.CDPERIENTR AND 
    	            PERFILPROD.CDPRODUTO  = (SELECT MAX(B.CDPRODUTO) 
    	        FROM 
    	            PERFILPROD A, 
    	            PRODUTO B 
    	        WHERE 
    	            A.CDPRODUTO = B.CDPRODUTO AND 
    	            A.CDFILIAL  = :P_CDFILIAL AND 
    	            INSTR(:P_CDPRODUTO, B.CDARVPROD) = 1)";
        try { 
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }  
    //abre tela Calendário de Compras se retornar vazio
    public function CalendarioDeCompras($row, $perientr)
    {
        $dataEntrega = $row['INTERVAL_DT']['start'];
        $params = array(
    	    "P_CDFILIAL"   => $row['CDFILIAL'],
    	    "P_CDPERIENTR" => $perientr[0]['CDPERIENTR'],
    	    "P_DTENTREGA"  => $dataEntrega
    	);
    	$sql = "SELECT 
    	            CDFILIAL, 
    	            CDPERIENTR, 
    	            DTENTREGA, 
    	            DTANTPROXENT 
    	        FROM 
    	            CALECOMP 
    	        WHERE 
    	            CDFILIAL   = :P_CDFILIAL   AND 
    	            CDPERIENTR = :P_CDPERIENTR AND 
    	            TO_CHAR(DTENTREGA, 'YYYY') = TO_CHAR(TO_DATE(:P_DTENTREGA), 'YYYY') 
    	        ORDER BY 
    	            DTENTREGA";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }    
    
    //Query para consulta de parâmetros das demais query´s
    public function ConsultaParametro($row) {
        $params = array(
    	    "P_CDFILIAL" => $row['CDFILIAL']
        );
    	$sql = "SELECT 
    	            NVL ( IDUTISTATCOT ,'C') AS IDUTISTATCOT, 
    	            NVL ( IDUTILCONSSOLIOL , 'N'), 
    	            NVL ( IDUTLPRARVOLCMP , 'N'), 
    	            NVL ( IDUTQTDISPGERAL , 'N'),
                    CASE
                        WHEN 
                            IDFUNCAOFILI = 'O' 
                        THEN 
                            'S'
                        ELSE 
                            'N' 
                    END, 
                    CDREGIAO, 
                    NVL(FILIAL.CDFILCENTCMP , FILIAL.CDFILIAL) AS CDFILCENTCMP
                FROM 
                    PARASUPRI P, 
                    FILIAL
                WHERE 
                    FILIAL.CDFILIAL = :P_CDFILIAL AND 
                    P.CDFILIAL      = NVL (FILIAL.CDFILCENTCMP , FILIAL.CDFILIAL )";
                
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    //Planilha de cotação de precos
    public function ConsultaMarcaPorProduto($row)
    {
        $params = array(
    	    "P_CDPRODUTO" => $row['CDPRODUTO'],
    	    "P_CDMARCA"   => $row['CDMARCA']
        );
    	$sql = "SELECT 
    	            COUNT(1) AS MARCAPROD 
    	        FROM 
    	            MARCPRODCOMP 
    	        WHERE 
    	            CDPRMARCOMPL = :P_CDPRODUTO AND 
    	            CDMARCA      = :P_CDMARCA";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaQualificacaoMarca($row, $ConsParam) 
    {
        $params = array(
             
            "P_CDFILCENTCMP" => $ConsParam[0]['CDFILCENTCMP'],
            "P_CDMARCA"      => $row['CDMARCA'],
            "P_DTINICIAL"    => $row['INTERVAL_DT']['start'],
            "P_CDPRODUTO"    => $row['CDPRODUTO']

        );
    	$sql = "SELECT COUNT(CDFILIAL) AS QLFMARCA
                FROM 
                    (SELECT PARASUPRI.CDFILIAL
                    FROM QFOMARPROD, PARASUPRI
                    WHERE (PARASUPRI.CDFILIAL = :P_CDFILCENTCMP)
                            AND (QFOMARPROD.CDMARCA = :P_CDMARCA)
                            AND ((QFOMARPROD.IDAPRMARPRO = 'N'
                            AND QFOMARPROD.IDAUTMARPRO = 'S')
                            OR ((QFOMARPROD.IDAPRMARPRO = 'S')
                            AND ((PARASUPRI.IDULTIDTVALQ = 'M'
                            AND :P_DTINICIAL <= LAST_DAY(QFOMARPROD.DTVALAPROVA))
                            OR (PARASUPRI.IDULTIDTVALQ = 'D'
                            AND :P_DTINICIAL <= QFOMARPROD.DTVALAPROVA))))
                    UNION
                    ALL SELECT PARASUPRI.CDFILIAL
                    FROM PARASUPRI
                    WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP
                            AND PARASUPRI.IDMARSEMQUA <> 'S'
                            AND NOT EXISTS 
                        (SELECT QFOMARPROD.CDMARCA
                        FROM QFOMARPROD
                        WHERE QFOMARPROD.CDPRODUTO = :P_CDPRODUTO
                                AND QFOMARPROD.CDMARCA = :P_CDMARCA))";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaProdutoPorFornecedor($row) 
    {
        $params = array(
    	    "P_CDFORNECED" => $row['CDFORNECED'],
    	    "P_CDPRODUTO"=> $row['CDPRODUTO']
        );
    	$sql = "SELECT 
    	            COUNT(1) AS PRODFORN
                FROM 
                    PRODFORNCOMP
                WHERE 
                    PRODFORNCOMP.CDFORNECED  = :P_CDFORNECED AND 
                    PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    } 
    
    public function ConsultaQualificacaoFornecedor($row, $ConsParam)
    {
        $params = array(
            "P_CDFILCENTCMP" => $ConsParam[0]['CDFILCENTCMP'],
            "P_CDFORNECED"   =>  $row['CDFORNECED'],
            "P_DTINICIAL"    => $row['INTERVAL_DT']['start']
        );
    	$sql = "SELECT COUNT(1) AS QLFFORN
                FROM 
                    (SELECT FORNECEDOR.CDFORNECED
                    FROM QUALIFICACAO, PARASUPRI, FORNECEDOR
                    WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP
                        AND FORNECEDOR.CDFORNECED = :P_CDFORNECED
                        AND FORNECEDOR.CDQUALIF = QUALIFICACAO.CDQUALIF
                        AND (QUALIFICACAO.IDPERCOTCMP <> '2'
                        AND (FORNECEDOR.IDFORNAUTO = 'S'
                        OR ((PARASUPRI.IDULTIDTVALQ = 'D'
                        AND :P_DTINICIAL < FORNECEDOR.DTVALQUALIF)
                        OR (PARASUPRI.IDULTIDTVALQ = 'M'
                        AND :P_DTINICIAL < LAST_DAY(FORNECEDOR.DTVALQUALIF)))))
                    UNION
                    ALL SELECT FORNECEDOR.CDFORNECED
                    FROM FORNECEDOR
                    WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED
                            AND FORNECEDOR.CDQUALIF IS NULL)";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaQualificacaoProdFornecedor($row, $ConsParam)
    {
        $params = array(
            "P_CDFILCENTCMP"=> $ConsParam[0]['CDFILCENTCMP'],
            "P_CDPRODUTO" => $row['CDPRODUTO'],
            "P_DTINICIAL" => $row['INTERVAL_DT']['start'],
            "P_CDFORNECED" =>  $row['CDFORNECED']
        );
    	$sql = "SELECT COUNT(1) AS QLFPRODFORN
                FROM DUAL
                WHERE NOT EXISTS 
                    (SELECT QFOPRODFORN.CDPRODUTO
                    FROM QFOPRODFORN, PARASUPRI
                    WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP
                            AND (QFOPRODFORN.CDPRODUTO = :P_CDPRODUTO)
                            AND (QFOPRODFORN.CDFORNECED = :P_CDFORNECED)
                            AND ((QFOPRODFORN.IDFORNAPROV = 'N'
                            AND QFOPRODFORN.IDFORNAUTOR = 'N')
                            OR ((QFOPRODFORN.IDFORNAPROV = 'S')
                            AND ((PARASUPRI.IDULTIDTVALQ = 'M'
                            AND :P_DTINICIAL > LAST_DAY(QFOPRODFORN.DTVALPRFORN))
                            OR (PARASUPRI.IDULTIDTVALQ = 'D'
                            AND :P_DTINICIAL > QFOPRODFORN.DTVALPRFORN)) ) ))";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaFornecedorNaoAtendeRegiao($row, $ConsParam)
    {
        $params = array(
              "P_CDREGIAO" =>  $ConsParam[0]['CDREGIAO'],
              "P_CDFORNECED" =>  $row['CDFORNECED']
        );
    	$sql = "SELECT COUNT(1) AS FORNREG
                FROM 
                    (SELECT '1'
                    FROM REFOREG
                    WHERE REFOREG.CDREGIAO = :P_CDREGIAO
                            AND REFOREG.CDFORNECED = :P_CDFORNECED
                            AND REFOREG.CDPERIENTR IS NOT NULL
                    UNION
                    ALL SELECT '1'
                    FROM DUAL
                    WHERE NOT EXISTS 
                        (SELECT REFOREG.CDREGIAO
                        FROM REFOREG
                        WHERE REFOREG.CDREGIAO = :P_CDREGIAO
                                AND REFOREG.CDFORNECED = :P_CDFORNECED))";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaFornecedorNaoAtendeFilial($row)
    {
        $params = array(
              "P_CDFILIAL"   =>  $row['CDFILIAL'],
              "P_CDFORNECED" =>  $row['CDFORNECED']
        );
    	$sql = "SELECT
                    COUNT(1) AS FORNFIL
                FROM
                    REFORFIL,
                    FILIAL
                WHERE
                    ( REFORFIL.CDFORNECED     = :P_CDFORNECED )
                    AND   ( REFORFIL.CDFILIAL = :P_CDFILIAL )
                    AND   ( REFORFIL.CDFILIAL = FILIAL.CDFILIAL )
                    AND   ( CDPERIENTR IS NULL )
                ORDER BY
                    REFORFIL.CDFILIAL";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaProdutoPorComprador($row)
    {
        $params = array(
              "P_CDPRODUTO"   =>  $row['CDPRODUTO'],
              "P_CDOPERADOR"  =>  $row['CDOPERADOR']
        );
    	$sql = "SELECT 1
                FROM PRODUTO
                WHERE PRODUTO.CDPRODUTO = :P_CDPRODUTO
                      AND NOT EXISTS (SELECT PRODCOMP.CDPRODUTO
                            FROM PRODCOMP INNER JOIN PRODUTO PR ON PRODCOMP.CDPRODUTO  = PR.CDPRODUTO
                           WHERE ((PRODCOMP.CDOPERADOR = :P_CDOPERADOR) OR (:P_CDOPERADOR = 'T'))
                            AND PRODUTO.CDARVPROD LIKE PR.CDARVPROD || '%')";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    //Apuração/Efetivação
    public function ConsultaRegiao($row)
    {
        $params = array(
            "P_CDFILIAL" =>  $row['CDFILIAL']
        );
    	$sql = "SELECT COUNT(1) AS REG
                FROM FILIAL
                WHERE (FILIAL.CDFILIAL = :P_CDFILIAL)
                        AND (FILIAL.CDREGIAO IS NOT NULL)";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    } 
    
/* CONSULTAS PARA Produto Preferencial/Quantidade Média Solicitada*/
    public function ConsultaProdPreferencial($row)
    {
        $params = array(
            "P_CDFILIAL"  => $row['CDFILIAL'],
            "P_CDPRODUTO" => $row['CDPRODUTO'],
        );
    	$sql = "SELECT COUNT(1) AS PRODPREF
                FROM PRODPREFMED, PARCMPSUP, PRODUTO
                WHERE PARCMPSUP.CDFILIAL = :P_CDFILIAL
                        AND PRODPREFMED.CDFILIAL = PARCMPSUP.CDFILIAL
                        AND PRODUTO.CDPRODUTO = :P_CDPRODUTO
                        AND PRODPREFMED.CDPRODUTO = PRODUTO.CDPRODESTO
                        AND NVL(PARCMPSUP.IDUTPRODPREF, 'N') = 'S'";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaQtdMedia($row)
    {
        $params = array(
            'P_CDFILIAL' => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
        );
        $sql = 'SELECT 
                    COUNT(1) AS QTDMEDIA
                FROM 
                    PRODPREFMED,
                    PRODUTO
                WHERE 
                    PRODUTO.CDPRODUTO      = :P_CDPRODUTO AND 
                    PRODPREFMED.CDPRODUTO  = PRODUTO.CDPRODESTO AND 
                    PRODPREFMED.CDFILIAL   = :P_CDFILIAL AND 
                    PRODPREFMED.CDPRODPREF <> :P_CDPRODUTO';
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaIdutistatcot($row, $ConsParam)
    {
        $params = array(
            "V_CDFILCENTCMP"  => $ConsParam[0]['CDFILCENTCMP'],
            "P_CDFORNECED"    => $row['CDFORNECED'],
            "P_CDREGIAO"      => $ConsParam[0]['CDREGIAO'],
            "P_CDPRODUTO"     => $row['CDPRODUTO'],
            "P_CDMARCA"       => $row['CDMARCA'],
            "P_DTINI"         => $row['INTERVAL_DT']['start'],
            "P_DTFIN"         => $row['INTERVAL_DT']['end']
        );
    	$sql = "SELECT COUNT(COLETAPR.IDSITUCOLETA) AS IDSITUCOLETA
                FROM COLETAPR, ITEMCOLE
                   WHERE ITEMCOLE.CDFILIAL      = :V_CDFILCENTCMP
                     AND ITEMCOLE.CDFORNECED    = :P_CDFORNECED
                     AND ITEMCOLE.CDREGIAO      = :P_CDREGIAO
                     AND ITEMCOLE.CDPRODUTO     = :P_CDPRODUTO
                     AND ITEMCOLE.CDMARCA       = :P_CDMARCA
                     AND ITEMCOLE.VRPRECUNIITM  <> 0
                     AND ITEMCOLE.DTINIVALITM BETWEEN CASE
                                                         WHEN ITEMCOLE.DTINIVALITM < TO_DATE(:P_DTINI)
                                                         THEN ITEMCOLE.DTINIVALITM
                                                         ELSE TO_DATE(:P_DTINI)
                                                      END AND TO_DATE(:P_DTFIN)
                     AND ITEMCOLE.DTFINVALITM BETWEEN TO_DATE(:P_DTINI) AND CASE
                                                                     WHEN ITEMCOLE.DTFINVALITM > TO_DATE(:P_DTFIN)
                                                                     THEN ITEMCOLE.DTFINVALITM
                                                                     ELSE TO_DATE(:P_DTFIN)
                                                                  END
                     AND COLETAPR.CDFILIAL      = ITEMCOLE.CDFILIAL
                     AND COLETAPR.NRCOLEPRECO   = ITEMCOLE.NRCOLEPRECO
                     AND NVL(COLETAPR.IDSITUCOLETA, 'A')  = 'C'";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    } 
    
    public function VerificarCustoLista ($row, $regiao) 
    {
        if (empty($regiao)) {
            $regiao = null;
        } else {
            $regiao =  $regiao[0]['CDREGIAO'];
        }
      
        $params = array(
              "P_CDREGIAO"     => $regiao,
              "P_CDPRODUTO"    => $row['CDPRODUTO'],
              "P_CDFILIAL"     => $row['CDFILIAL'],
              "P_CDFILCENTCMP" => $row['CDFILCENTCMP'],
              "P_NRSEQPRELIST" => $row['NRSEQPRELIST'],
              "P_CDMARCA"      => $row['CDMARCA'],
              "P_CDFORNECED"   => $row['CDFORNECED']
        );

    	$sql = "SELECT
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        PRODPREFMED P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = PRODUTO.CDPRODESTO
                        AND   P.CDPRODPREF = ITEMCOLE.CDPRODUTO
                    UNION
                    SELECT
                        '1'
                    FROM
                        DUAL
                    WHERE
                        NOT EXISTS(
                            SELECT
                                '1'
                            FROM
                                PRODPREFMED
                            WHERE
                                PRODPREFMED.CDFILIAL = FILIAL.CDFILIAL
                                AND   PRODPREFMED.CDPRODUTO = PRODUTO.CDPRODESTO
                        )
                ),NULL,'PREF','1') ) IDPRODPREF,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        REFORFIL R
                    WHERE
                        R.CDFILIAL = FILIAL.CDFILIAL
                        AND   R.CDFORNECED = ITEMCOLE.CDFORNECED
                        AND   R.CDPERIENTR IS NULL
                ),NULL,'1','FNF') ) IDREFORFIL,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        REFOREG
                    WHERE
                        REFOREG.CDREGIAO = ITEMCOLE.CDREGIAO
                        AND   REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED
                ),NULL,'1','FNR') ) IDREFOREG,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        QFOPRODFORN,
                        PARASUPRI
                    WHERE
                        PARASUPRI.CDFILIAL = ITEMCOLE.CDFILIAL
                        AND   QFOPRODFORN.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   QFOPRODFORN.CDFORNECED = ITEMCOLE.CDFORNECED
                        AND   (
                            (
                                QFOPRODFORN.IDFORNAPROV = 'N'
                                AND   QFOPRODFORN.IDFORNAUTOR = 'N'
                            )
                            OR    (
                                (QFOPRODFORN.IDFORNAPROV = 'S')
                                AND   (
                                    (
                                        PARASUPRI.IDULTIDTVALQ = 'M'
                                        AND   COLETAPR.DTCOLEPREC > LAST_DAY(QFOPRODFORN.DTVALPRFORN)
                                    )
                                    OR    (
                                        PARASUPRI.IDULTIDTVALQ = 'D'
                                        AND   COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN
                                    )
                                )
                            )
                        )
                ),NULL,'1','QPF') ) IDQFOPRODFORN,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        QFOMARPROD,
                        PARASUPRI
                    WHERE
                        PARASUPRI.CDFILIAL = ITEMCOLE.CDFILIAL
                        AND   QFOMARPROD.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   QFOMARPROD.CDMARCA = ITEMCOLE.CDMARCA
                        AND   (
                            (
                                QFOMARPROD.IDAPRMARPRO = 'N'
                                AND   QFOMARPROD.IDAUTMARPRO = 'N'
                            )
                            OR    (
                                (QFOMARPROD.IDAPRMARPRO = 'S')
                                AND   (
                                    (
                                        PARASUPRI.IDULTIDTVALQ = 'M'
                                        AND   COLETAPR.DTCOLEPREC > LAST_DAY(QFOMARPROD.DTVALAPROVA)
                                    )
                                    OR    (
                                        PARASUPRI.IDULTIDTVALQ = 'D'
                                        AND   COLETAPR.DTCOLEPREC > QFOMARPROD.DTVALAPROVA
                                    )
                                )
                            )
                        )
                ),NULL,'1','QM') ) IDQFOMARPROD,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        QFOMARPROD
                    WHERE
                        QFOMARPROD.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   QFOMARPROD.CDMARCA = ITEMCOLE.CDMARCA
                    UNION
                    SELECT
                        '1'
                    FROM
                        PARASUPRI
                    WHERE
                        PARASUPRI.CDFILIAL = ITEMCOLE.CDFILIAL
                        AND   PARASUPRI.IDMARSEMQUA = 'N'
                        OR    PARASUPRI.IDMARSEMQUA IS NULL
                ),NULL,'QM','1') ) IDQFOPARASUPRI,
                MIN(DECODE( (
                    SELECT
                        '1'
                    FROM
                        FORNECEDOR F,
                        QUALIFICACAO,
                        PARASUPRI
                    WHERE
                        F.CDFORNECED = ITEMCOLE.CDFORNECED
                        AND   F.IDSITUCADA = 'A'
                        AND   NVL(F.IDTIPOFORN,'N') <> 'A'
                        AND   QUALIFICACAO.CDQUALIF(+) = F.CDQUALIF
                        AND   PARASUPRI.CDFILIAL = ITEMCOLE.CDFILIAL
                        AND   (
                            (NVL(QUALIFICACAO.IDPERCOTCMP(+),'1') = '1')
                            AND   (
                                (
                                    PARASUPRI.IDULTIDTVALQ = 'D'
                                    AND   COLETAPR.DTCOLEPREC <= NVL(F.DTVALQUALIF,COLETAPR.DTCOLEPREC)
                                )
                                OR    (
                                    PARASUPRI.IDULTIDTVALQ = 'M'
                                    AND   COLETAPR.DTCOLEPREC <= LAST_DAY(NVL(F.DTVALQUALIF,COLETAPR.DTCOLEPREC) )
                                )
                            )
                        )
                ),NULL,'CF','1') ) IDFORNECEDOR,
                MIN(DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART = '3'
                        AND   P.CDMARCA = ITEMCOLE.CDMARCA
                ),NULL,'1','RM') ) IDRESTROMARCA,
                MIN(DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART = '6'
                        AND   P.CDFORNECED = ITEMCOLE.CDFORNECED
                ),NULL,'1','RF') ) IDRESTRFORN,
                MIN(DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART = '9'
                        AND   P.CDMARCA = ITEMCOLE.CDMARCA
                        AND   P.CDFORNECED = ITEMCOLE.CDFORNECED
                ),NULL,'1','RMF') ) IDRESTRMARCFORN,
                MIN(DECODE(INSTR( (DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART IN(
                            '1','2'
                        )
                        AND   P.CDMARCA = ITEMCOLE.CDMARCA
                ),NULL,'0','1')
                || DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.NRCOLEPRECO IS NULL
                        AND   P.CDTIPPART IN(
                            '4','5'
                        )
                        AND   P.CDFORNECED = ITEMCOLE.CDFORNECED
                ),NULL,'0','1')
                || DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART IN(
                            '7','8'
                        )
                        AND   P.CDMARCA = ITEMCOLE.CDMARCA
                        AND   P.CDFORNECED = ITEMCOLE.CDFORNECED
                ),NULL,'0','1')
                || DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                        AND   P.CDTIPPART IN(
                            '3','6','9'
                        )
                        AND   NOT EXISTS(
                            SELECT
                                '1'
                            FROM
                                PARTFILI PART
                            WHERE
                                (PART.CDFILIAL = FILIAL.CDFILIAL)
                                AND   (PART.CDPRODUTO = ITEMCOLE.CDPRODUTO)
                                AND   (PART.NRCOLEPRECO IS NULL)
                                AND   (PART.CDTIPPART IN(
                                    '1','2','4','5','7','8'
                                ) )
                        )
                ),NULL,'0','1')
                || DECODE( (
                    SELECT DISTINCT
                        '1'
                    FROM
                        PARTFILI P
                    WHERE
                        P.CDFILIAL = FILIAL.CDFILIAL
                        AND   P.NRCOLEPRECO IS NULL
                        AND   P.CDPRODUTO = ITEMCOLE.CDPRODUTO
                ),NULL,'1','0') ),'1'),0,'0','1') ) IDSITUPART,
                REGIAO.CDREGIAO,
                REGIAO.NMREGIAO,
                FILIAL.CDFILIAL,
                FILIAL.NMFILIAL,
                CDARVPROD,
                NMPRODUTO,
                ITEMCOLE.NRORG
            FROM
                COLETAPR,
                PRODUTO,
                FILIAL,
                ITEMCOLE,
                REGIAO
            WHERE
                ITEMCOLE.CDREGIAO =:P_CDREGIAO
                AND   ITEMCOLE.CDREGIAO = REGIAO.CDREGIAO
                AND   PRODUTO.CDPRODUTO =:P_CDPRODUTO
                AND   FILIAL.CDFILIAL =:P_CDFILIAL
                AND   ITEMCOLE.CDFILIAL =:P_CDFILCENTCMP
                AND   ITEMCOLE.NRSEQPRELIST =:P_NRSEQPRELIST
                AND   ITEMCOLE.IDITMLISPREC = 'S'
                AND   ITEMCOLE.VRPRECUNIITM > 0
                AND   ITEMCOLE.CDREGIAO = FILIAL.CDREGIAO
                AND   ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO
                AND   ITEMCOLE.CDFORNECED = NVL(:P_CDFORNECED,ITEMCOLE.CDFORNECED)
                AND   ITEMCOLE.CDMARCA = NVL(:P_CDMARCA,ITEMCOLE.CDMARCA)
                AND   COLETAPR.CDFILIAL = ITEMCOLE.CDFILIAL
                AND   COLETAPR.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO
            GROUP BY
                REGIAO.CDREGIAO,
                REGIAO.NMREGIAO,
                FILIAL.CDFILIAL,
                FILIAL.NMFILIAL,
                CDARVPROD,
                NMPRODUTO,
                ITEMCOLE.NRORG";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    } 
    
    public function getOperadorLista($row)
    {
        $params = array(
            'P_CDFILIAL'   => $row['CDFILIAL'],
            "P_DTINICIAL"  => $row['INTERVAL_DT']['start'],
            'P_DTFINIAL'   => $row['INTERVAL_DT']['end'],
            'NRSEQPRELIST' => $row['NRSEQPRELIST'],
            'P_TVALIDADE'  => $row['P_TVALIDADE']
        );
        $sql = "SELECT PREVLISTA.CDOPERADOR ,
                  PREVLISTA.NRORG
                FROM PREVLISTA, FILIAL
                WHERE FILIAL.CDFILIAL          = :P_CDFILIAL
                AND PREVLISTA.NRORG            = FILIAL.NRORG
                AND PREVLISTA.CDFILIAL         = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                AND PREVLISTA.IDDISPLISFIL     = 'S'
                AND (((PREVLISTA.DTINIVALLIST <= :P_DTFINIAL)
                AND (PREVLISTA.DTFINVALLIST   >= :P_DTINICIAL))
                OR (:P_TVALIDADE               = 'T'))
                AND PREVLISTA.NRSEQPRELIST     = :NRSEQPRELIST ";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
}
